from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .address_1 import Address1
from .dispute import Dispute
from .payment_ewallets import PaymentEwallets
from .payment_instructions import PaymentInstructions
from .next_action import NextAction
from .outcome import Outcome
from .fee import Fee
from .payment_refunds import PaymentRefunds
from .payment_status import PaymentStatus


class PaymentMethodTypeCategory(Enum):
    """An enumeration representing different categories.

    :cvar BANKTRANSFER: "bank_transfer"
    :vartype BANKTRANSFER: str
    :cvar BANKREDIRECT: "bank_redirect"
    :vartype BANKREDIRECT: str
    :cvar CARD: "card"
    :vartype CARD: str
    :cvar CASH: "cash"
    :vartype CASH: str
    :cvar EWALLET: "ewallet"
    :vartype EWALLET: str
    """

    BANKTRANSFER = "bank_transfer"
    BANKREDIRECT = "bank_redirect"
    CARD = "card"
    CASH = "cash"
    EWALLET = "ewallet"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, PaymentMethodTypeCategory._member_map_.values())
        )


@JsonMap({"id_": "id"})
class Payment(BaseModel):
    """Collects money from a payment method and deposits it into one or more Rapyd Wallets

    :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
    :type address: Address1, optional
    :param amount: The amount received by the recipient, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. To verify a card, set to 0., defaults to None
    :type amount: float, optional
    :param auth_code: Indicates that the card payment was authorized by the card network., defaults to None
    :type auth_code: str, optional
    :param cancel_reason: Reason for cancellation or reversal of a payment. Response only., defaults to None
    :type cancel_reason: str, optional
    :param captured: Indicates whether the payment has been successfully captured. Response only., defaults to None
    :type captured: bool, optional
    :param complete_payment_url: URL where the customer is redirected for final steps in completing the operation. Provided by the clientRelevant to bank redirect payment methods, defaults to None
    :type complete_payment_url: str, optional
    :param country_code: country_code, defaults to None
    :type country_code: str, optional
    :param created_at: Time of creation of the payment, in Unix time. Response only., defaults to None
    :type created_at: int, optional
    :param currency_code: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency_code: str, optional
    :param customer_token: ID of the customer who is making the payment. String starting with **cus_**. When a payment is created without an existing customer, the platform creates an anonymous customer with no payment methods. Response only., defaults to None
    :type customer_token: str, optional
    :param description: Description of the payment, defaults to None
    :type description: str, optional
    :param dispute: Describes the fields contained in REST messages and webhooks for disputes of payments., defaults to None
    :type dispute: Dispute, optional
    :param error_code: Relevant error message (with an underscore between words) and ID number of the error. Response only., defaults to None
    :type error_code: str, optional
    :param error_payment_url: URL where the customer is redirected in case of an error in the operation. Provided by the client.Relevant to bank redirect payment methods, defaults to None
    :type error_payment_url: str, optional
    :param escrow: Describes the escrow. Relevant when the payment is created with escrow set to true. Response only., defaults to None
    :type escrow: dict, optional
    :param ewallet_id: ID of the wallet that the money is paid into. String starting with ewallet_. Relevant when the request includes a single wallet. Response only., defaults to None
    :type ewallet_id: str, optional
    :param ewallets: Specifies the wallets that the money is collected into. If this is left blank, the money goes into the oldest 'collection'-type client wallet. If there is no 'collection' client wallet, the money goes into the oldest 'general'-type client wallet., defaults to None
    :type ewallets: List[PaymentEwallets], optional
    :param expiration: End of the time allowed for customer to make this payment, in Unix time. Must be after the current time, defaults to None
    :type expiration: float, optional
    :param failure_code: Error code explaining the reason for failure of the payment. Response only., defaults to None
    :type failure_code: str, optional
    :param failure_message: Message to the merchant, explaining the reason for failure of the payment. Response only., defaults to None
    :type failure_message: str, optional
    :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer). * buy - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD. This is the default. * sell - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
    :type fixed_side: str, optional
    :param flow_type: Reserved., defaults to None
    :type flow_type: str, optional
    :param fx_rate: Exchange rate for the transaction. When fixed_side is buy, fx_rate is the buy rate. When fixed_side is sell, fx_rate is the sell rate. Decimal number as string. Response only., defaults to None
    :type fx_rate: float, optional
    :param group_payment: ID of the group payment. String starting with gp_. Relevant when the payment is part of a group payment., defaults to None
    :type group_payment: str, optional
    :param id_: ID of the payment. String starting with payment_. Response only, defaults to None
    :type id_: str, optional
    :param initiation_type: Reason for cancellation or reversal of a payment. Response only., defaults to None
    :type initiation_type: str, optional
    :param instructions: Describes how the customer makes the payment. Read-only. Contains the following fields - * name - Description of the payment method. * steps - A steps object containing a list of steps for the customer to take. Each step is named stepN, where N is an integer., defaults to None
    :type instructions: List[PaymentInstructions], optional
    :param invoice: ID of the invoice that this payment is for. String starting with inv_. Response only., defaults to None
    :type invoice: str, optional
    :param is_partial: Indicates whether the payment has been partially paid. When false, indicates that the payment is unpaid or fully paid. Response only., defaults to None
    :type is_partial: bool, optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param merchant_requested_amount: Indicates the amount paid by the payer, in units of the currency defined in merchant_requested_currency. Relevant to payments with FX. Read only., defaults to None
    :type merchant_requested_amount: str, optional
    :param merchant_requested_currency: Indicates the currency that merchant receives. Three-letter ISO 4217 code. Uppercase. Relevant to payments with FX. Response only., defaults to None
    :type merchant_requested_currency: str, optional
    :param metadata: A JSON object defined by the client in the Customer Payment Method object. Response only., defaults to None
    :type metadata: dict, optional
    :param mid: Reserved, defaults to None
    :type mid: str, optional
    :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
    :type next_action: NextAction, optional
    :param order: ID of the order that this payment is for. Read-only. Relevant when the payment is for an order., defaults to None
    :type order: str, optional
    :param original_amount: * Foreign exchange payments - The amount paid by the sender, in units of the currency defined in requested_currency, including gross transaction fees and gross FX fees. * Payments not involving foreign exchange - The amount of the payment, in units of the currency defined in currency, including gross transaction fees. Response only., defaults to None
    :type original_amount: float, optional
    :param outcome: The outcome object describes the outcome of the Rapyd Protect risk assessment, defaults to None
    :type outcome: Outcome, optional
    :param paid: Indicates whether the payment has been fully captured. Response only., defaults to None
    :type paid: bool, optional
    :param paid_at: Time of the last capture, in Unix time. Response only., defaults to None
    :type paid_at: float, optional
    :param payment_fees: payment_fees, defaults to None
    :type payment_fees: Fee, optional
    :param payment_method: payment_method ID or object. If not specified in this field, the payment method is the default payment method specified for the customer. Mandatory when there is no default payment method. For a description of the fields in the payment_method object, see Customer Payment Method Object., defaults to None
    :type payment_method: str, optional
    :param payment_method_data: Details of the payment_method_data object. See Payment Method Data Object. Response only., defaults to None
    :type payment_method_data: dict, optional
    :param payment_method_options: Object describing additional payment method fields required for the payment. These values are not saved as part of the payment method object. To determine the fields required, run Get Payment Method Required Fields., defaults to None
    :type payment_method_options: dict, optional
    :param payment_method_type: A type of payment method that a customer can use for making payments.  The payment method type has a suffix with one of the following values [_bank, _card, _cash, _ewallet], defaults to None
    :type payment_method_type: str, optional
    :param payment_method_type_category: Category of payment method type. Read-only., defaults to None
    :type payment_method_type_category: PaymentMethodTypeCategory, optional
    :param receipt_email: Email address that the receipt for this transaction is sent to, defaults to None
    :type receipt_email: str, optional
    :param receipt_number: Reserved. Response only, defaults to None
    :type receipt_number: str, optional
    :param redirect_url: URL where the customer is redirected for additional steps required for the payment. Response only. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication, defaults to None
    :type redirect_url: str, optional
    :param refunded: Indicates whether there was a refund against this payment. Response only., defaults to None
    :type refunded: bool, optional
    :param refunded_amount: The total amount refunded against this payment, in units of the currency defined in currency. Response only., defaults to None
    :type refunded_amount: float, optional
    :param refunds: Refunds object, defaults to None
    :type refunds: PaymentRefunds, optional
    :param remitter_information: Contains the customer name and the associated bank account. This includes - * name - Name of the customer. * account_id - ID of the customer's bank account. * bank_code - SWIFT code for the customer's bank. Response only., defaults to None
    :type remitter_information: dict, optional
    :param save_payment_method: * true - Saves the card for future use. This is the default. * false - Does not save the card. Relevant when payment_method_type_category is card and the "Create Payment" request includes full card details., defaults to None
    :type save_payment_method: bool, optional
    :param statement_descriptor: A text description suitable for a customer's payment statement. Limited to 22 characters. If this field is not specified, Rapyd populates it with the name of the merchant, defaults to None
    :type statement_descriptor: str, optional
    :param status: status, defaults to None
    :type status: PaymentStatus, optional
    :param textual_codes: A set of text codes for the customer to use to complete the steps described in the instructions field. Response only.   The name of the field is the local name of the code, or some other label. For example   * code * paycode * pay_code * payid * pairing_code * payment_code * response_code, defaults to None
    :type textual_codes: dict, optional
    :param transaction_id: ID of the associated transaction. Response only, defaults to None
    :type transaction_id: str, optional
    :param visual_codes: A set of images for the customer to use to complete the steps described in the instructions field. For example, a QR code or barcode. Response only., defaults to None
    :type visual_codes: dict, optional
    """

    def __init__(
        self,
        address: Address1 = None,
        amount: float = None,
        auth_code: str = None,
        cancel_reason: str = None,
        captured: bool = None,
        complete_payment_url: str = None,
        country_code: str = None,
        created_at: int = None,
        currency_code: str = None,
        customer_token: str = None,
        description: str = None,
        dispute: Dispute = None,
        error_code: str = None,
        error_payment_url: str = None,
        escrow: dict = None,
        ewallet_id: str = None,
        ewallets: List[PaymentEwallets] = None,
        expiration: float = None,
        failure_code: str = None,
        failure_message: str = None,
        fixed_side: str = None,
        flow_type: str = None,
        fx_rate: float = None,
        group_payment: str = None,
        id_: str = None,
        initiation_type: str = None,
        instructions: List[PaymentInstructions] = None,
        invoice: str = None,
        is_partial: bool = None,
        merchant_reference_id: str = None,
        merchant_requested_amount: str = None,
        merchant_requested_currency: str = None,
        metadata: dict = None,
        mid: str = None,
        next_action: NextAction = None,
        order: str = None,
        original_amount: float = None,
        outcome: Outcome = None,
        paid: bool = None,
        paid_at: float = None,
        payment_fees: Fee = None,
        payment_method: str = None,
        payment_method_data: dict = None,
        payment_method_options: dict = None,
        payment_method_type: str = None,
        payment_method_type_category: PaymentMethodTypeCategory = None,
        receipt_email: str = None,
        receipt_number: str = None,
        redirect_url: str = None,
        refunded: bool = None,
        refunded_amount: float = None,
        refunds: PaymentRefunds = None,
        remitter_information: dict = None,
        save_payment_method: bool = None,
        statement_descriptor: str = None,
        status: PaymentStatus = None,
        textual_codes: dict = None,
        transaction_id: str = None,
        visual_codes: dict = None,
    ):
        """Collects money from a payment method and deposits it into one or more Rapyd Wallets

        :param address: address associated with this specific Rapyd entity Payment/Customer etc., defaults to None
        :type address: Address1, optional
        :param amount: The amount received by the recipient, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. To verify a card, set to 0., defaults to None
        :type amount: float, optional
        :param auth_code: Indicates that the card payment was authorized by the card network., defaults to None
        :type auth_code: str, optional
        :param cancel_reason: Reason for cancellation or reversal of a payment. Response only., defaults to None
        :type cancel_reason: str, optional
        :param captured: Indicates whether the payment has been successfully captured. Response only., defaults to None
        :type captured: bool, optional
        :param complete_payment_url: URL where the customer is redirected for final steps in completing the operation. Provided by the clientRelevant to bank redirect payment methods, defaults to None
        :type complete_payment_url: str, optional
        :param country_code: country_code, defaults to None
        :type country_code: str, optional
        :param created_at: Time of creation of the payment, in Unix time. Response only., defaults to None
        :type created_at: int, optional
        :param currency_code: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency_code: str, optional
        :param customer_token: ID of the customer who is making the payment. String starting with **cus_**. When a payment is created without an existing customer, the platform creates an anonymous customer with no payment methods. Response only., defaults to None
        :type customer_token: str, optional
        :param description: Description of the payment, defaults to None
        :type description: str, optional
        :param dispute: Describes the fields contained in REST messages and webhooks for disputes of payments., defaults to None
        :type dispute: Dispute, optional
        :param error_code: Relevant error message (with an underscore between words) and ID number of the error. Response only., defaults to None
        :type error_code: str, optional
        :param error_payment_url: URL where the customer is redirected in case of an error in the operation. Provided by the client.Relevant to bank redirect payment methods, defaults to None
        :type error_payment_url: str, optional
        :param escrow: Describes the escrow. Relevant when the payment is created with escrow set to true. Response only., defaults to None
        :type escrow: dict, optional
        :param ewallet_id: ID of the wallet that the money is paid into. String starting with ewallet_. Relevant when the request includes a single wallet. Response only., defaults to None
        :type ewallet_id: str, optional
        :param ewallets: Specifies the wallets that the money is collected into. If this is left blank, the money goes into the oldest 'collection'-type client wallet. If there is no 'collection' client wallet, the money goes into the oldest 'general'-type client wallet., defaults to None
        :type ewallets: List[PaymentEwallets], optional
        :param expiration: End of the time allowed for customer to make this payment, in Unix time. Must be after the current time, defaults to None
        :type expiration: float, optional
        :param failure_code: Error code explaining the reason for failure of the payment. Response only., defaults to None
        :type failure_code: str, optional
        :param failure_message: Message to the merchant, explaining the reason for failure of the payment. Response only., defaults to None
        :type failure_message: str, optional
        :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer). * buy - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD. This is the default. * sell - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
        :type fixed_side: str, optional
        :param flow_type: Reserved., defaults to None
        :type flow_type: str, optional
        :param fx_rate: Exchange rate for the transaction. When fixed_side is buy, fx_rate is the buy rate. When fixed_side is sell, fx_rate is the sell rate. Decimal number as string. Response only., defaults to None
        :type fx_rate: float, optional
        :param group_payment: ID of the group payment. String starting with gp_. Relevant when the payment is part of a group payment., defaults to None
        :type group_payment: str, optional
        :param id_: ID of the payment. String starting with payment_. Response only, defaults to None
        :type id_: str, optional
        :param initiation_type: Reason for cancellation or reversal of a payment. Response only., defaults to None
        :type initiation_type: str, optional
        :param instructions: Describes how the customer makes the payment. Read-only. Contains the following fields - * name - Description of the payment method. * steps - A steps object containing a list of steps for the customer to take. Each step is named stepN, where N is an integer., defaults to None
        :type instructions: List[PaymentInstructions], optional
        :param invoice: ID of the invoice that this payment is for. String starting with inv_. Response only., defaults to None
        :type invoice: str, optional
        :param is_partial: Indicates whether the payment has been partially paid. When false, indicates that the payment is unpaid or fully paid. Response only., defaults to None
        :type is_partial: bool, optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param merchant_requested_amount: Indicates the amount paid by the payer, in units of the currency defined in merchant_requested_currency. Relevant to payments with FX. Read only., defaults to None
        :type merchant_requested_amount: str, optional
        :param merchant_requested_currency: Indicates the currency that merchant receives. Three-letter ISO 4217 code. Uppercase. Relevant to payments with FX. Response only., defaults to None
        :type merchant_requested_currency: str, optional
        :param metadata: A JSON object defined by the client in the Customer Payment Method object. Response only., defaults to None
        :type metadata: dict, optional
        :param mid: Reserved, defaults to None
        :type mid: str, optional
        :param next_action: Indicates the next action for completing the payment. Response only. One of the following values are - * 3d_verification - The next action is 3DS authentication. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication. Relevant only to card payments. * pending_capture - The next action is pending the capture of the amount. Relevant only to card payments when the amount is not zero. * pending_confirmation - The next action is pending the confirmation for the payment. Relevant to all payment methods excluding card payment. * not_applicable - The payment has completed or the next action is not relevant., defaults to None
        :type next_action: NextAction, optional
        :param order: ID of the order that this payment is for. Read-only. Relevant when the payment is for an order., defaults to None
        :type order: str, optional
        :param original_amount: * Foreign exchange payments - The amount paid by the sender, in units of the currency defined in requested_currency, including gross transaction fees and gross FX fees. * Payments not involving foreign exchange - The amount of the payment, in units of the currency defined in currency, including gross transaction fees. Response only., defaults to None
        :type original_amount: float, optional
        :param outcome: The outcome object describes the outcome of the Rapyd Protect risk assessment, defaults to None
        :type outcome: Outcome, optional
        :param paid: Indicates whether the payment has been fully captured. Response only., defaults to None
        :type paid: bool, optional
        :param paid_at: Time of the last capture, in Unix time. Response only., defaults to None
        :type paid_at: float, optional
        :param payment_fees: payment_fees, defaults to None
        :type payment_fees: Fee, optional
        :param payment_method: payment_method ID or object. If not specified in this field, the payment method is the default payment method specified for the customer. Mandatory when there is no default payment method. For a description of the fields in the payment_method object, see Customer Payment Method Object., defaults to None
        :type payment_method: str, optional
        :param payment_method_data: Details of the payment_method_data object. See Payment Method Data Object. Response only., defaults to None
        :type payment_method_data: dict, optional
        :param payment_method_options: Object describing additional payment method fields required for the payment. These values are not saved as part of the payment method object. To determine the fields required, run Get Payment Method Required Fields., defaults to None
        :type payment_method_options: dict, optional
        :param payment_method_type: A type of payment method that a customer can use for making payments.  The payment method type has a suffix with one of the following values [_bank, _card, _cash, _ewallet], defaults to None
        :type payment_method_type: str, optional
        :param payment_method_type_category: Category of payment method type. Read-only., defaults to None
        :type payment_method_type_category: PaymentMethodTypeCategory, optional
        :param receipt_email: Email address that the receipt for this transaction is sent to, defaults to None
        :type receipt_email: str, optional
        :param receipt_number: Reserved. Response only, defaults to None
        :type receipt_number: str, optional
        :param redirect_url: URL where the customer is redirected for additional steps required for the payment. Response only. To simulate 3DS authentication in the sandbox, see Simulating 3DS Authentication, defaults to None
        :type redirect_url: str, optional
        :param refunded: Indicates whether there was a refund against this payment. Response only., defaults to None
        :type refunded: bool, optional
        :param refunded_amount: The total amount refunded against this payment, in units of the currency defined in currency. Response only., defaults to None
        :type refunded_amount: float, optional
        :param refunds: Refunds object, defaults to None
        :type refunds: PaymentRefunds, optional
        :param remitter_information: Contains the customer name and the associated bank account. This includes - * name - Name of the customer. * account_id - ID of the customer's bank account. * bank_code - SWIFT code for the customer's bank. Response only., defaults to None
        :type remitter_information: dict, optional
        :param save_payment_method: * true - Saves the card for future use. This is the default. * false - Does not save the card. Relevant when payment_method_type_category is card and the "Create Payment" request includes full card details., defaults to None
        :type save_payment_method: bool, optional
        :param statement_descriptor: A text description suitable for a customer's payment statement. Limited to 22 characters. If this field is not specified, Rapyd populates it with the name of the merchant, defaults to None
        :type statement_descriptor: str, optional
        :param status: status, defaults to None
        :type status: PaymentStatus, optional
        :param textual_codes: A set of text codes for the customer to use to complete the steps described in the instructions field. Response only.   The name of the field is the local name of the code, or some other label. For example   * code * paycode * pay_code * payid * pairing_code * payment_code * response_code, defaults to None
        :type textual_codes: dict, optional
        :param transaction_id: ID of the associated transaction. Response only, defaults to None
        :type transaction_id: str, optional
        :param visual_codes: A set of images for the customer to use to complete the steps described in the instructions field. For example, a QR code or barcode. Response only., defaults to None
        :type visual_codes: dict, optional
        """
        self.address = self._define_object(address, Address1)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.auth_code = self._define_str("auth_code", auth_code, nullable=True)
        self.cancel_reason = self._define_str(
            "cancel_reason", cancel_reason, nullable=True
        )
        self.captured = captured
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.country_code = self._define_str(
            "country_code",
            country_code,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency_code = self._define_str(
            "currency_code",
            currency_code,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer_token = self._define_str(
            "customer_token", customer_token, nullable=True
        )
        self.description = self._define_str("description", description, nullable=True)
        self.dispute = self._define_object(dispute, Dispute)
        self.error_code = self._define_str("error_code", error_code, nullable=True)
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.escrow = escrow
        self.ewallet_id = self._define_str("ewallet_id", ewallet_id, nullable=True)
        self.ewallets = self._define_list(ewallets, PaymentEwallets)
        self.expiration = self._define_number("expiration", expiration, nullable=True)
        self.failure_code = self._define_str(
            "failure_code", failure_code, nullable=True
        )
        self.failure_message = self._define_str(
            "failure_message", failure_message, nullable=True
        )
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.flow_type = self._define_str("flow_type", flow_type, nullable=True)
        self.fx_rate = self._define_number("fx_rate", fx_rate, nullable=True)
        self.group_payment = self._define_str(
            "group_payment", group_payment, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.initiation_type = self._define_str(
            "initiation_type", initiation_type, nullable=True
        )
        self.instructions = self._define_list(instructions, PaymentInstructions)
        self.invoice = self._define_str("invoice", invoice, nullable=True)
        self.is_partial = is_partial
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.merchant_requested_amount = self._define_str(
            "merchant_requested_amount", merchant_requested_amount, nullable=True
        )
        self.merchant_requested_currency = self._define_str(
            "merchant_requested_currency", merchant_requested_currency, nullable=True
        )
        self.metadata = metadata
        self.mid = self._define_str("mid", mid, nullable=True)
        self.next_action = (
            self._enum_matching(next_action, NextAction.list(), "next_action")
            if next_action
            else None
        )
        self.order = self._define_str("order", order, nullable=True)
        self.original_amount = self._define_number(
            "original_amount", original_amount, nullable=True
        )
        self.outcome = self._define_object(outcome, Outcome)
        self.paid = paid
        self.paid_at = self._define_number("paid_at", paid_at, nullable=True)
        self.payment_fees = self._define_object(payment_fees, Fee)
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )
        self.payment_method_data = payment_method_data
        self.payment_method_options = payment_method_options
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.payment_method_type_category = (
            self._enum_matching(
                payment_method_type_category,
                PaymentMethodTypeCategory.list(),
                "payment_method_type_category",
            )
            if payment_method_type_category
            else None
        )
        self.receipt_email = self._define_str(
            "receipt_email", receipt_email, nullable=True
        )
        self.receipt_number = self._define_str(
            "receipt_number", receipt_number, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.refunded = refunded
        self.refunded_amount = self._define_number(
            "refunded_amount", refunded_amount, nullable=True
        )
        self.refunds = self._define_object(refunds, PaymentRefunds)
        self.remitter_information = remitter_information
        self.save_payment_method = save_payment_method
        self.statement_descriptor = self._define_str(
            "statement_descriptor", statement_descriptor, nullable=True
        )
        self.status = (
            self._enum_matching(status, PaymentStatus.list(), "status")
            if status
            else None
        )
        self.textual_codes = textual_codes
        self.transaction_id = self._define_str(
            "transaction_id", transaction_id, nullable=True
        )
        self.visual_codes = visual_codes



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"type_": "type"})
class Limit(BaseModel):
    """Limit

    :param amount: The amount of the limit., defaults to None
    :type amount: float, optional
    :param currency: Three-letter ISO 4217 code for the currency of an existing account., defaults to None
    :type currency: str, optional
    :param type_: The limit type., defaults to None
    :type type_: str, optional
    :param updated_at: updated_at, defaults to None
    :type updated_at: int, optional
    """

    def __init__(
        self,
        amount: float = None,
        currency: str = None,
        type_: str = None,
        updated_at: int = None,
    ):
        """Limit

        :param amount: The amount of the limit., defaults to None
        :type amount: float, optional
        :param currency: Three-letter ISO 4217 code for the currency of an existing account., defaults to None
        :type currency: str, optional
        :param type_: The limit type., defaults to None
        :type type_: str, optional
        :param updated_at: updated_at, defaults to None
        :type updated_at: int, optional
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .list_webhooks_response_attempts_http_response_headers import (
    ListWebhooksResponseAttemptsHttpResponseHeaders,
)


@JsonMap({})
class ListWebhooksResponseAttempts(BaseModel):
    """List of attempts at sending the webhook. Array of objects that contain the following fields:

    :param error: Error message received from the configured webhook destination., defaults to None
    :type error: str, optional
    :param http_status_code: HTTP status code returned by the configured webhook destination., defaults to None
    :type http_status_code: str, optional
    :param http_response_body: Body of the HTTP response from the configured webhook destination., defaults to None
    :type http_response_body: str, optional
    :param http_response_headers: Headers of the HTTP response from the configured webhook destination, with the value received., defaults to None
    :type http_response_headers: ListWebhooksResponseAttemptsHttpResponseHeaders, optional
    """

    def __init__(
        self,
        error: str = None,
        http_status_code: str = None,
        http_response_body: str = None,
        http_response_headers: ListWebhooksResponseAttemptsHttpResponseHeaders = None,
    ):
        """List of attempts at sending the webhook. Array of objects that contain the following fields:

        :param error: Error message received from the configured webhook destination., defaults to None
        :type error: str, optional
        :param http_status_code: HTTP status code returned by the configured webhook destination., defaults to None
        :type http_status_code: str, optional
        :param http_response_body: Body of the HTTP response from the configured webhook destination., defaults to None
        :type http_response_body: str, optional
        :param http_response_headers: Headers of the HTTP response from the configured webhook destination, with the value received., defaults to None
        :type http_response_headers: ListWebhooksResponseAttemptsHttpResponseHeaders, optional
        """
        self.error = self._define_str("error", error, nullable=True)
        self.http_status_code = self._define_str(
            "http_status_code", http_status_code, nullable=True
        )
        self.http_response_body = self._define_str(
            "http_response_body", http_response_body, nullable=True
        )
        self.http_response_headers = self._define_object(
            http_response_headers, ListWebhooksResponseAttemptsHttpResponseHeaders
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class CollectPaymentsBody(BaseModel):
    """CollectPaymentsBody

    :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> On the hosted payment page, the customer: \<BR\> * Cannot modify the amount when amount_is_editable is false or not used. \<BR\> * Can modify the amount when amount_is_editable is true and amount has a positive value. \<BR\> * Must enter an amount when amount_is_editable is true and amount is 0, null, or not used.
    :type amount: str
    :param amount_is_editable: Determines whether the customer can edit the amount. \<BR\> * **true** - The customer can edit the amount. \<BR\> * **false** - The customer cannot edit the amount., defaults to None
    :type amount_is_editable: bool, optional
    :param checkout: Optional parameters for the checkout page., defaults to None
    :type checkout: dict, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country.
    :type country: str
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields.
    :type currency: str
    :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer., defaults to None
    :type customer: str, optional
    :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer).\<BR\>* **buy** - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD.\<BR\>* **sell** - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
    :type fixed_side: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'. \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param max_payments: Indicates the maximum number of times that the payment link can be used for payments. When not indicated, there is no limit., defaults to None
    :type max_payments: float, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param requested_currency: Currency for one side of an FX transaction. Three-letter ISO 4217 code. \<BR\>\<BR\>* When `fixed_side` is **sell**, it is the currency received in the Rapyd Wallet. \<BR\>\<BR\>* When `fixed_side` is **buy**, it is the currency charged to the buyer (customer)., defaults to None
    :type requested_currency: str, optional
    """

    def __init__(
        self,
        amount: str,
        country: str,
        currency: str,
        amount_is_editable: bool = None,
        checkout: dict = None,
        customer: str = None,
        fixed_side: str = None,
        language: str = None,
        max_payments: float = None,
        merchant_reference_id: str = None,
        requested_currency: str = None,
    ):
        """CollectPaymentsBody

        :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\> On the hosted payment page, the customer: \<BR\> * Cannot modify the amount when amount_is_editable is false or not used. \<BR\> * Can modify the amount when amount_is_editable is true and amount has a positive value. \<BR\> * Must enter an amount when amount_is_editable is true and amount is 0, null, or not used.
        :type amount: str
        :param amount_is_editable: Determines whether the customer can edit the amount. \<BR\> * **true** - The customer can edit the amount. \<BR\> * **false** - The customer cannot edit the amount., defaults to None
        :type amount_is_editable: bool, optional
        :param checkout: Optional parameters for the checkout page., defaults to None
        :type checkout: dict, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country.
        :type country: str
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields.
        :type currency: str
        :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer., defaults to None
        :type customer: str, optional
        :param fixed_side: Indicates whether the FX rate is fixed for the buy side (seller) or for the sell side (buyer).\<BR\>* **buy** - The currency that the Rapyd Wallet receives for goods or services. Fixed side buy relates to the seller (merchant) funds. For example, a US-based merchant wants to charge 100 USD. The buyer (customer) pays the amount in MXN that converts to 100 USD.\<BR\>* **sell** - The currency that the buyer is charged for purchasing goods or services. Fixed side sell relates to the buyer (customer) funds. For example, a US-based merchant wants to charge a buyer 2,000 MXN and will accept whatever amount in USD that is converted from 2,000 MXN., defaults to None
        :type fixed_side: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'. \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param max_payments: Indicates the maximum number of times that the payment link can be used for payments. When not indicated, there is no limit., defaults to None
        :type max_payments: float, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param requested_currency: Currency for one side of an FX transaction. Three-letter ISO 4217 code. \<BR\>\<BR\>* When `fixed_side` is **sell**, it is the currency received in the Rapyd Wallet. \<BR\>\<BR\>* When `fixed_side` is **buy**, it is the currency charged to the buyer (customer)., defaults to None
        :type requested_currency: str, optional
        """
        self.amount = amount
        self.amount_is_editable = amount_is_editable
        self.checkout = checkout
        self.country = country
        self.currency = currency
        self.customer = self._define_str("customer", customer, nullable=True)
        self.fixed_side = self._define_str("fixed_side", fixed_side, nullable=True)
        self.language = self._define_str("language", language, nullable=True)
        self.max_payments = self._define_number(
            "max_payments", max_payments, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_24,
    InlineResponse200_25,
    InvoiceItemsInvoiceItemBody,
    V1InvoiceItemsBody,
)


class SubscriptionInvoiceItemService(BaseService):

    @cast_models
    def list_invoice_items(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        created: str = None,
        customer: str = None,
        invoice: str = None,
        limit: str = None,
        ending_before: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_24:
        """Retrieve a list of all invoice items. <BR> You can filter the list with query parameters.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param created: Date of creation of the invoice items., defaults to None
        :type created: str, optional
        :param customer: ID of the customer., defaults to None
        :type customer: str, optional
        :param invoice: ID of the invoice., defaults to None
        :type invoice: str, optional
        :param limit: The maximum number of invoice items to return. Range: 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param ending_before: The ID of the invoice item created after the last invoice item you want to retrieve., defaults to None
        :type ending_before: str, optional
        :param starting_after: The ID of the invoice item created before the first invoice item you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice items listed successfully
        :rtype: InlineResponse200_24
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(created)
        Validator(str).is_optional().validate(customer)
        Validator(str).is_optional().validate(invoice)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/invoice_items", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("created", created, explode=False)
            .add_query("customer", customer, explode=False)
            .add_query("invoice", invoice, explode=False)
            .add_query("limit", limit, explode=False)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_24._unmap(response)

    @cast_models
    def create_invoice_item(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: V1InvoiceItemsBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_24:
        """Create an invoice item and add it to an invoice or subscription.<BR>**Note**: If you create an invoice item without specifying the invoice ID or subscription ID, it is attached to the customer’s next invoice that has the same currency.<BR> This method triggers the following webhooks:<BR> * **Invoice Item Created** - This webhook contains the same information as the response. <BR>* Invoice Updated Webhook

        :param request_body: The request body., defaults to None
        :type request_body: V1InvoiceItemsBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice item created successfully
        :rtype: InlineResponse200_24
        """

        Validator(V1InvoiceItemsBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/invoice_items", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_24._unmap(response)

    @cast_models
    def get_invoice_item(
        self,
        invoice_item: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_24:
        """Retrieve the details of an invoice item.

        :param invoice_item: Unique access key provided by Rapyd for each authorized user.
        :type invoice_item: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice item retrieved successfully
        :rtype: InlineResponse200_24
        """

        Validator(str).validate(invoice_item)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoice_items/{{invoiceItem}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceItem", invoice_item)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_24._unmap(response)

    @cast_models
    def update_invoice_item(
        self,
        invoice_item: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: InvoiceItemsInvoiceItemBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_24:
        """Change or modify an invoice item.<BR> You can update an invoice item at any time before the corresponding subscription generates an invoice.<BR> This method triggers the **Invoice Item Updated** webhook. This webhook contains the same information as the response.

        :param request_body: The request body., defaults to None
        :type request_body: InvoiceItemsInvoiceItemBody, optional
        :param invoice_item: Unique access key provided by Rapyd for each authorized user.
        :type invoice_item: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice item updated successfully
        :rtype: InlineResponse200_24
        """

        Validator(InvoiceItemsInvoiceItemBody).is_optional().validate(request_body)
        Validator(str).validate(invoice_item)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoice_items/{{invoiceItem}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceItem", invoice_item)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_24._unmap(response)

    @cast_models
    def delete_invoice_item(
        self,
        invoice_item: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_25:
        """Delete an invoice item from the upcoming invoice.<BR> Use this method in the following situations:<BR> * The invoice item is attached to an invoice.<BR> * The invoice item is not attached to an invoice.<BR> This method triggers the **Invoice Item Deleted webhook**. This webhook contains the same information as the response.Retrieve the details of an invoice item.

        :param invoice_item: Unique access key provided by Rapyd for each authorized user.
        :type invoice_item: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Invoice item deleted successfully.
        :rtype: InlineResponse200_25
        """

        Validator(str).validate(invoice_item)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/invoice_items/{{invoiceItem}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("invoiceItem", invoice_item)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_25._unmap(response)



from typing import Union
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.one_of_base_model import OneOfBaseModel


class ConditionThresholdValueGuard(OneOfBaseModel):
    class_list = {"str": str, "int": int}


ConditionThresholdValue = Union[str, int]


@JsonMap({})
class Condition(BaseModel):
    """Condition

    :param description: Description of the condition, defaults to None
    :type description: str, optional
    :param element_name: The name of a field, including the path. The field is the first operand of the condition. The path starts with one of the following - * payment - The field is a Create Payment body parameter. Not relevant to the payment_method and payment_method_options objects. * payment.payment_method_type - The field appears in the current Get Payment Method Required Fields response under fields or payment_method_options. * organization - The field relates to your organization, such as your merchant type category. Your organization specifies the field when activating your account. See Activating Your Account (KYB), defaults to None
    :type element_name: str, optional
    :param operator: A symbol representing the operator of the condition. String starting with $. The operator determines the relationship between the operands See documentation - "https://docs.rapyd.net/en/payment-method-type.html", defaults to None
    :type operator: str, optional
    :param threshold_value: One or more possible values of the element_name field. The second operand of the condition "https://docs.rapyd.net/en/payment-method-type.html", defaults to None
    :type threshold_value: ConditionThresholdValue, optional
    """

    def __init__(
        self,
        description: str = None,
        element_name: str = None,
        operator: str = None,
        threshold_value: ConditionThresholdValue = None,
    ):
        """Condition

        :param description: Description of the condition, defaults to None
        :type description: str, optional
        :param element_name: The name of a field, including the path. The field is the first operand of the condition. The path starts with one of the following - * payment - The field is a Create Payment body parameter. Not relevant to the payment_method and payment_method_options objects. * payment.payment_method_type - The field appears in the current Get Payment Method Required Fields response under fields or payment_method_options. * organization - The field relates to your organization, such as your merchant type category. Your organization specifies the field when activating your account. See Activating Your Account (KYB), defaults to None
        :type element_name: str, optional
        :param operator: A symbol representing the operator of the condition. String starting with $. The operator determines the relationship between the operands See documentation - "https://docs.rapyd.net/en/payment-method-type.html", defaults to None
        :type operator: str, optional
        :param threshold_value: One or more possible values of the element_name field. The second operand of the condition "https://docs.rapyd.net/en/payment-method-type.html", defaults to None
        :type threshold_value: ConditionThresholdValue, optional
        """
        self.description = self._define_str("description", description, nullable=True)
        self.element_name = self._define_str(
            "element_name", element_name, nullable=True
        )
        self.operator = self._define_str("operator", operator, nullable=True)
        self.threshold_value = ConditionThresholdValueGuard.return_one_of(
            threshold_value
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1hostedcollectcard_card_fields import V1hostedcollectcardCardFields


@JsonMap({})
class CollectCardBody(BaseModel):
    """CollectCardBody

    :param billing_address_collect: Indicates whether the hosted page displays address fields to fill in.\<BR\> * **true** - The address fields are displayed. \<BR\> * **false** - The address fields are displayed only for country codes **US**, **GB** and **CA**., defaults to None
    :type billing_address_collect: bool, optional
    :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param card_fields: Contains details about the card., defaults to None
    :type card_fields: V1hostedcollectcardCardFields, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs.The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
    :type complete_url: str, optional
    :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
    :type complete_payment_url: str, optional
    :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country.
    :type country: str
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields., defaults to None
    :type currency: str, optional
    :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer.
    :type customer: str
    :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
    :type error_payment_url: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. Range: 1 minute to 30 days., defaults to None
    :type page_expiration: str, optional
    :param payment_method_type: Limits the page to a specific type of payment method. For example, **dk_visa_card**. For a list of payment methods, use [List Payment Methods by Country](https://docs.rapyd.net/en/list-payment-methods-by-country.html)., defaults to None
    :type payment_method_type: str, optional
    """

    def __init__(
        self,
        country: str,
        customer: str,
        billing_address_collect: bool = None,
        cancel_url: str = None,
        card_fields: V1hostedcollectcardCardFields = None,
        complete_url: str = None,
        complete_payment_url: str = None,
        currency: str = None,
        error_payment_url: str = None,
        language: str = None,
        page_expiration: str = None,
        payment_method_type: str = None,
    ):
        """CollectCardBody

        :param billing_address_collect: Indicates whether the hosted page displays address fields to fill in.\<BR\> * **true** - The address fields are displayed. \<BR\> * **false** - The address fields are displayed only for country codes **US**, **GB** and **CA**., defaults to None
        :type billing_address_collect: bool, optional
        :param cancel_url: URL where the customer is redirected after pressing **Back to Website** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param card_fields: Contains details about the card., defaults to None
        :type card_fields: V1hostedcollectcardCardFields, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs.The two-letter ISO 3166-1 ALPHA-2 code for the country., defaults to None
        :type complete_url: str, optional
        :param complete_payment_url: URL where the customer is redirected when payment is successful, after returning from an external page such as a 3DS page. Does not support localhost URLs., defaults to None
        :type complete_payment_url: str, optional
        :param country: The two-letter ISO 3166-1 ALPHA-2 code for the country.
        :type country: str
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code. \<BR\> In FX transactions: \<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet. \<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer. \<BR\> See also `fixed_side` and `requested_currency` fields., defaults to None
        :type currency: str, optional
        :param customer: ID of a specific customer. String starting with **cus_**. Restricts the payment link to the customer.
        :type customer: str
        :param error_payment_url: URL where the customer is redirected when payment is not successful, after returning from an external page, such as a 3DS page. Does not support localhost URLs., defaults to None
        :type error_payment_url: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see [List Supported Languages](https://docs.rapyd.net/en/list-supported-languages.html). \<BR\> * When this parameter is null, the language of the user's browser is used. \<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. Range: 1 minute to 30 days., defaults to None
        :type page_expiration: str, optional
        :param payment_method_type: Limits the page to a specific type of payment method. For example, **dk_visa_card**. For a list of payment methods, use [List Payment Methods by Country](https://docs.rapyd.net/en/list-payment-methods-by-country.html)., defaults to None
        :type payment_method_type: str, optional
        """
        self.billing_address_collect = billing_address_collect
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.card_fields = self._define_object(
            card_fields, V1hostedcollectcardCardFields
        )
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.complete_payment_url = self._define_str(
            "complete_payment_url", complete_payment_url, nullable=True
        )
        self.country = country
        self.currency = self._define_str("currency", currency, nullable=True)
        self.customer = customer
        self.error_payment_url = self._define_str(
            "error_payment_url", error_payment_url, nullable=True
        )
        self.language = self._define_str("language", language, nullable=True)
        self.page_expiration = self._define_str(
            "page_expiration", page_expiration, nullable=True
        )
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_12,
    InlineResponse200_13,
    InlineResponse200_14,
    ProductsProductsIdBody,
    V1ProductsBody,
)


class SubscriptionProductService(BaseService):

    @cast_models
    def get_products_list(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        ending_before: str = None,
        limit: str = None,
        starting_after: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_13:
        """Retrieve a list of all products.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param ending_before: The ID of the products created after the last product you want to retrieve., defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of products to return. Range 1-100. Default is 10., defaults to None
        :type limit: str, optional
        :param starting_after: The ID of the product created before the first products you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of products
        :rtype: InlineResponse200_13
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/products", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("limit", limit, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_13._unmap(response)

    @cast_models
    def create_product(
        self,
        request_body: V1ProductsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_14:
        """Create goods or services.

        :param request_body: The request body.
        :type request_body: V1ProductsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created Product
        :rtype: InlineResponse200_14
        """

        Validator(V1ProductsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/products", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_14._unmap(response)

    @cast_models
    def get_product(
        self,
        products_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_14:
        """Retrieve details of a product or service.

        :param products_id: ID of the product.
        :type products_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The Product
        :rtype: InlineResponse200_14
        """

        Validator(str).validate(products_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/products/{{productsId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("productsId", products_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_14._unmap(response)

    @cast_models
    def update_product(
        self,
        products_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: ProductsProductsIdBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_14:
        """Change or modify a product or service.

        :param request_body: The request body., defaults to None
        :type request_body: ProductsProductsIdBody, optional
        :param products_id: ID of the product.
        :type products_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created Product
        :rtype: InlineResponse200_14
        """

        Validator(ProductsProductsIdBody).is_optional().validate(request_body)
        Validator(str).validate(products_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/products/{{productsId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("productsId", products_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_14._unmap(response)

    @cast_models
    def delete_product(
        self,
        products_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_12:
        """Delete a product or service from the Rapyd platform. This action also deletes all associated plans.

        :param products_id: ID of the product.
        :type products_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Rapyd response with operation result, and product id
        :rtype: InlineResponse200_12
        """

        Validator(str).validate(products_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/products/{{productsId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("productsId", products_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_12._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class RefundsCompleteBody(BaseModel):
    """RefundsCompleteBody

    :param token: ID of the refund. String starting with **refund_**.
    :type token: str
    """

    def __init__(self, token: str):
        """RefundsCompleteBody

        :param token: ID of the refund. String starting with **refund_**.
        :type token: str
        """
        self.token = token



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class ListSupportedLanguagesResponseLanguages(BaseModel):
    """List of the languages supported for hosted pages.

    :param name: Name of the language in English., defaults to None
    :type name: str, optional
    :param iso_alpha2: ISO 639-1 code for the language. 2 letters, with suffix where relevant., defaults to None
    :type iso_alpha2: str, optional
    """

    def __init__(self, name: str = None, iso_alpha2: str = None):
        """List of the languages supported for hosted pages.

        :param name: Name of the language in English., defaults to None
        :type name: str, optional
        :param iso_alpha2: ISO 639-1 code for the language. 2 letters, with suffix where relevant., defaults to None
        :type iso_alpha2: str, optional
        """
        self.name = self._define_str("name", name, nullable=True)
        self.iso_alpha2 = self._define_str("iso_alpha2", iso_alpha2, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .contact import Contact


@JsonMap({"type_": "type"})
class V1ewalletsContact(BaseModel):
    """V1ewalletsContact

    :param data: data, defaults to None
    :type data: List[Contact], optional
    :param total_count: total_count, defaults to None
    :type total_count: float, optional
    :param has_more: has_more, defaults to None
    :type has_more: bool, optional
    :param url: url, defaults to None
    :type url: str, optional
    :param ewallet_reference_id: Rapyd Wallet ID defined by the customer or end user. Must be unique., defaults to None
    :type ewallet_reference_id: str, optional
    :param first_name: First name of the Rapyd Wallet owner., defaults to None
    :type first_name: str, optional
    :param last_name: Last name of the Rapyd Wallet owner., defaults to None
    :type last_name: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param type_: Type of wallet, company or person. Default is person., defaults to None
    :type type_: str, optional
    :param phone_number: Phone number of the Rapyd Wallet owner., defaults to None
    :type phone_number: str, optional
    """

    def __init__(
        self,
        data: List[Contact] = None,
        total_count: float = None,
        has_more: bool = None,
        url: str = None,
        ewallet_reference_id: str = None,
        first_name: str = None,
        last_name: str = None,
        metadata: dict = None,
        type_: str = None,
        phone_number: str = None,
    ):
        """V1ewalletsContact

        :param data: data, defaults to None
        :type data: List[Contact], optional
        :param total_count: total_count, defaults to None
        :type total_count: float, optional
        :param has_more: has_more, defaults to None
        :type has_more: bool, optional
        :param url: url, defaults to None
        :type url: str, optional
        :param ewallet_reference_id: Rapyd Wallet ID defined by the customer or end user. Must be unique., defaults to None
        :type ewallet_reference_id: str, optional
        :param first_name: First name of the Rapyd Wallet owner., defaults to None
        :type first_name: str, optional
        :param last_name: Last name of the Rapyd Wallet owner., defaults to None
        :type last_name: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param type_: Type of wallet, company or person. Default is person., defaults to None
        :type type_: str, optional
        :param phone_number: Phone number of the Rapyd Wallet owner., defaults to None
        :type phone_number: str, optional
        """
        self.data = self._define_list(data, Contact)
        self.total_count = self._define_number(
            "total_count", total_count, nullable=True
        )
        self.has_more = has_more
        self.url = self._define_str("url", url, nullable=True)
        self.ewallet_reference_id = self._define_str(
            "ewallet_reference_id", ewallet_reference_id, nullable=True
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.metadata = metadata
        self.type_ = self._define_str("type_", type_, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class BalanceHoldBody(BaseModel):
    """BalanceHoldBody

    :param amount: Amount of the transfer. Decimal.
    :type amount: float
    :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
    :type currency: str, optional
    :param ewallet: ID of the wallet associated with the contact. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    """

    def __init__(self, amount: float, currency: str = None, ewallet: str = None):
        """BalanceHoldBody

        :param amount: Amount of the transfer. Decimal.
        :type amount: float
        :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field., defaults to None
        :type currency: str, optional
        :param ewallet: ID of the wallet associated with the contact. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        """
        self.amount = amount
        self.currency = self._define_str("currency", currency, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class RequestedCurrency(Enum):
    """An enumeration representing different categories.

    :cvar AUD: "AUD"
    :vartype AUD: str
    :cvar EUR: "EUR"
    :vartype EUR: str
    :cvar GBP: "GBP"
    :vartype GBP: str
    :cvar HKD: "HKD"
    :vartype HKD: str
    :cvar SGD: "SGD"
    :vartype SGD: str
    :cvar USD: "USD"
    :vartype USD: str
    """

    AUD = "AUD"
    EUR = "EUR"
    GBP = "GBP"
    HKD = "HKD"
    SGD = "SGD"
    USD = "USD"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, RequestedCurrency._member_map_.values()))


class InlineResponse200_103DataBankAccountsStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar CLO: "CLO"
    :vartype CLO: str
    :cvar ERR: "ERR"
    :vartype ERR: str
    :cvar REJ: "REJ"
    :vartype REJ: str
    """

    ACT = "ACT"
    CLO = "CLO"
    ERR = "ERR"
    REJ = "REJ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                InlineResponse200_103DataBankAccountsStatus._member_map_.values(),
            )
        )


@JsonMap({})
class InlineResponse200_103DataBankAccounts(BaseModel):
    """Array of objects with details of the virtual accounts. Each object contains the following fields

    :param account_id: The actual account number that was assigned to the virtual account when it was created., defaults to None
    :type account_id: str, optional
    :param account_id_type: Type of the virtual account number, such as IBAN or CLABE., defaults to None
    :type account_id_type: str, optional
    :param country_iso: Two-letter ISO 3166-1 ALPHA-2 code of the country of the virtual account., defaults to None
    :type country_iso: str, optional
    :param currency: Currency of the virtual account. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param ewallet: ID of the Rapyd Wallet that the virtual accounts were issued to. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param issuing_id: ID of the virtual account number object. String starting with **issuing_**., defaults to None
    :type issuing_id: str, optional
    :param requested_currency: Currency received by the virtual account after conversion.\<BR\>When not specified, the funds appear in the wallet’s currency account for the currency of the transaction., defaults to None
    :type requested_currency: RequestedCurrency, optional
    :param status: Indicates the status of the virtual account., defaults to None
    :type status: InlineResponse200_103DataBankAccountsStatus, optional
    """

    def __init__(
        self,
        account_id: str = None,
        account_id_type: str = None,
        country_iso: str = None,
        currency: str = None,
        ewallet: str = None,
        issuing_id: str = None,
        requested_currency: RequestedCurrency = None,
        status: InlineResponse200_103DataBankAccountsStatus = None,
    ):
        """Array of objects with details of the virtual accounts. Each object contains the following fields

        :param account_id: The actual account number that was assigned to the virtual account when it was created., defaults to None
        :type account_id: str, optional
        :param account_id_type: Type of the virtual account number, such as IBAN or CLABE., defaults to None
        :type account_id_type: str, optional
        :param country_iso: Two-letter ISO 3166-1 ALPHA-2 code of the country of the virtual account., defaults to None
        :type country_iso: str, optional
        :param currency: Currency of the virtual account. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param ewallet: ID of the Rapyd Wallet that the virtual accounts were issued to. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param issuing_id: ID of the virtual account number object. String starting with **issuing_**., defaults to None
        :type issuing_id: str, optional
        :param requested_currency: Currency received by the virtual account after conversion.\<BR\>When not specified, the funds appear in the wallet’s currency account for the currency of the transaction., defaults to None
        :type requested_currency: RequestedCurrency, optional
        :param status: Indicates the status of the virtual account., defaults to None
        :type status: InlineResponse200_103DataBankAccountsStatus, optional
        """
        self.account_id = self._define_str("account_id", account_id, nullable=True)
        self.account_id_type = self._define_str(
            "account_id_type", account_id_type, nullable=True
        )
        self.country_iso = self._define_str("country_iso", country_iso, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.issuing_id = self._define_str("issuing_id", issuing_id, nullable=True)
        self.requested_currency = (
            self._enum_matching(
                requested_currency, RequestedCurrency.list(), "requested_currency"
            )
            if requested_currency
            else None
        )
        self.status = (
            self._enum_matching(
                status, InlineResponse200_103DataBankAccountsStatus.list(), "status"
            )
            if status
            else None
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_31Data(BaseModel):
    """InlineResponse200_31Data

    :param account_name: Name of the bank account holder., defaults to None
    :type account_name: str, optional
    :param account_number: Number of the bank account., defaults to None
    :type account_number: str, optional
    :param amount: The amount of the transaction, in units of the currency defined in currency., defaults to None
    :type amount: str, optional
    :param bank_code: Bank identifier code, such as SWIFT, BIC, ABA, or other local identifier., defaults to None
    :type bank_code: str, optional
    :param bank_name: Name of the bank., defaults to None
    :type bank_name: str, optional
    :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
    :type country: str, optional
    :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
    :type created_at: float, optional
    :param currency: Currency of the account. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param iban: IBAN of the virtual bank account., defaults to None
    :type iban: str, optional
    :param id_: ID that is assigned by the remitter, defaults to None
    :type id_: str, optional
    :param partner_transaction_id: ID that is assigned by the client to the transaction., defaults to None
    :type partner_transaction_id: str, optional
    :param receiving_iban: IBAN that was assigned to the Rapyd Wallet., defaults to None
    :type receiving_iban: str, optional
    :param remitter_reference: Optional information that the remitter adds to the transaction, intended for the beneficiary. Supported by some payment methods., defaults to None
    :type remitter_reference: str, optional
    :param sort_code: sort_code, defaults to None
    :type sort_code: str, optional
    :param status: Indicates the status of the transfer., defaults to None
    :type status: str, optional
    :param updated_at: Time of update of the virtual account, in Unix time. Response only., defaults to None
    :type updated_at: float, optional
    :param created_at_gw: Time of creation of the transaction, in Unix time. Response only., defaults to None
    :type created_at_gw: str, optional
    :param original_currency: This field will display the currency in which the transaction was originally made., defaults to None
    :type original_currency: str, optional
    :param original_amount: This field will show the initial transaction amount before conversion., defaults to None
    :type original_amount: str, optional
    :param fx_rate: This field will indicate the exchange rate applied for the currency conversion, defaults to None
    :type fx_rate: str, optional
    """

    def __init__(
        self,
        account_name: str = None,
        account_number: str = None,
        amount: str = None,
        bank_code: str = None,
        bank_name: str = None,
        country: str = None,
        created_at: float = None,
        currency: str = None,
        iban: str = None,
        id_: str = None,
        partner_transaction_id: str = None,
        receiving_iban: str = None,
        remitter_reference: str = None,
        sort_code: str = None,
        status: str = None,
        updated_at: float = None,
        created_at_gw: str = None,
        original_currency: str = None,
        original_amount: str = None,
        fx_rate: str = None,
    ):
        """InlineResponse200_31Data

        :param account_name: Name of the bank account holder., defaults to None
        :type account_name: str, optional
        :param account_number: Number of the bank account., defaults to None
        :type account_number: str, optional
        :param amount: The amount of the transaction, in units of the currency defined in currency., defaults to None
        :type amount: str, optional
        :param bank_code: Bank identifier code, such as SWIFT, BIC, ABA, or other local identifier., defaults to None
        :type bank_code: str, optional
        :param bank_name: Name of the bank., defaults to None
        :type bank_name: str, optional
        :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
        :type country: str, optional
        :param created_at: Time of creation of the transaction, in Unix time. Response only., defaults to None
        :type created_at: float, optional
        :param currency: Currency of the account. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param iban: IBAN of the virtual bank account., defaults to None
        :type iban: str, optional
        :param id_: ID that is assigned by the remitter, defaults to None
        :type id_: str, optional
        :param partner_transaction_id: ID that is assigned by the client to the transaction., defaults to None
        :type partner_transaction_id: str, optional
        :param receiving_iban: IBAN that was assigned to the Rapyd Wallet., defaults to None
        :type receiving_iban: str, optional
        :param remitter_reference: Optional information that the remitter adds to the transaction, intended for the beneficiary. Supported by some payment methods., defaults to None
        :type remitter_reference: str, optional
        :param sort_code: sort_code, defaults to None
        :type sort_code: str, optional
        :param status: Indicates the status of the transfer., defaults to None
        :type status: str, optional
        :param updated_at: Time of update of the virtual account, in Unix time. Response only., defaults to None
        :type updated_at: float, optional
        :param created_at_gw: Time of creation of the transaction, in Unix time. Response only., defaults to None
        :type created_at_gw: str, optional
        :param original_currency: This field will display the currency in which the transaction was originally made., defaults to None
        :type original_currency: str, optional
        :param original_amount: This field will show the initial transaction amount before conversion., defaults to None
        :type original_amount: str, optional
        :param fx_rate: This field will indicate the exchange rate applied for the currency conversion, defaults to None
        :type fx_rate: str, optional
        """
        self.account_name = self._define_str(
            "account_name", account_name, nullable=True
        )
        self.account_number = self._define_str(
            "account_number", account_number, nullable=True
        )
        self.amount = self._define_str("amount", amount, nullable=True)
        self.bank_code = self._define_str("bank_code", bank_code, nullable=True)
        self.bank_name = self._define_str("bank_name", bank_name, nullable=True)
        self.country = self._define_str("country", country, nullable=True)
        self.created_at = self._define_number("created_at", created_at, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.iban = self._define_str("iban", iban, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.partner_transaction_id = self._define_str(
            "partner_transaction_id", partner_transaction_id, nullable=True
        )
        self.receiving_iban = self._define_str(
            "receiving_iban", receiving_iban, nullable=True
        )
        self.remitter_reference = self._define_str(
            "remitter_reference", remitter_reference, nullable=True
        )
        self.sort_code = self._define_str("sort_code", sort_code, nullable=True)
        self.status = self._define_str("status", status, nullable=True)
        self.updated_at = self._define_number("updated_at", updated_at, nullable=True)
        self.created_at_gw = self._define_str(
            "created_at_gw", created_at_gw, nullable=True
        )
        self.original_currency = self._define_str(
            "original_currency", original_currency, nullable=True
        )
        self.original_amount = self._define_str(
            "original_amount", original_amount, nullable=True
        )
        self.fx_rate = self._define_str("fx_rate", fx_rate, nullable=True)



from enum import Enum


class Gender(Enum):
    """An enumeration representing different categories.

    :cvar MALE: "male"
    :vartype MALE: str
    :cvar FEMALE: "female"
    :vartype FEMALE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar NOTAPPLICABLE: "not_applicable"
    :vartype NOTAPPLICABLE: str
    """

    MALE = "male"
    FEMALE = "female"
    OTHER = "other"
    NOTAPPLICABLE = "not_applicable"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Gender._member_map_.values()))



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .hosted_page_additional_response_cart_items import (
    HostedPageAdditionalResponseCartItems,
)
from .hosted_page_additional_response_custom_elements import (
    HostedPageAdditionalResponseCustomElements,
)
from .payment import Payment


@JsonMap({"id_": "id"})
class V1CheckoutBody(BaseModel):
    """V1CheckoutBody

    :param account_funding_transaction: Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
    :type account_funding_transaction: dict, optional
    :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal.
    :type amount: float
    :param cancel_checkout_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type cancel_checkout_url: str, optional
    :param cart_items: Describes the cart items that the customer is purchasing. These items are displayed at the checkout page., defaults to None
    :type cart_items: HostedPageAdditionalResponseCartItems, optional
    :param complete_checkout_url: URL where the customer is redirected after pressing Finish to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_checkout_url: str, optional
    :param country: country
    :type country: str
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
    :type currency: str
    :param customer: Description of the payment transaction. To display the description, set display_description to true in custom_elements., defaults to None
    :type customer: str, optional
    :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
    :type escrow: bool, optional
    :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range: 1-90., defaults to None
    :type escrow_release_days: float, optional
    :param id_: ID of the Rapyd checkout page. String starting with **checkout_**., defaults to None
    :type id_: str, optional
    :param merchant_main_button: A string that represents the text on the main Call to Action (CTA) button. One of the following:\<BR\>* place_your_order - Place Your Order.\<BR\>* pay - Pay.\<BR\>* pay_now - Pay Now.\<BR\>* make_payment - Make Payment.\<BR\>* purchase - Purchase.\<BR\>* buy - Buy.\<BR\>* donate - Donate.\<BR\>* To configure this button, use the Client Portal., defaults to None
    :type merchant_main_button: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
    :type merchant_terms: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the Client Portal, defaults to None
    :type merchant_website: str, optional
    :param custom_elements: Description of the payment transaction., defaults to None
    :type custom_elements: HostedPageAdditionalResponseCustomElements, optional
    :param page_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
    :type page_expiration: float, optional
    :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
    :type payment: Payment, optional
    :param payment_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
    :type payment_expiration: float, optional
    :param payment_method_type: The type of the payment method. For example, **it_visa_card**, defaults to None
    :type payment_method_type: str, optional
    :param payment_method_type_categories: A list of the categories of payment method that are supported on the checkout page. The categories appear on the page in the order provided, defaults to None
    :type payment_method_type_categories: List[str], optional
    :param payment_method_types_exclude: List of payment methods that are excluded from display on the checkout page., defaults to None
    :type payment_method_types_exclude: List[str], optional
    :param payment_method_types_include: List of payment methods that are displayed on the checkout page. The payment methods appear on the page in the order provided., defaults to None
    :type payment_method_types_include: List[str], optional
    :param timestamp: Time of creation of the checkout page, in Unix time., defaults to None
    :type timestamp: float, optional
    """

    def __init__(
        self,
        amount: float,
        country: str,
        currency: str,
        account_funding_transaction: dict = None,
        cancel_checkout_url: str = None,
        cart_items: HostedPageAdditionalResponseCartItems = None,
        complete_checkout_url: str = None,
        customer: str = None,
        escrow: bool = None,
        escrow_release_days: float = None,
        id_: str = None,
        merchant_main_button: str = None,
        merchant_privacy_policy: str = None,
        merchant_terms: str = None,
        merchant_website: str = None,
        custom_elements: HostedPageAdditionalResponseCustomElements = None,
        page_expiration: float = None,
        payment: Payment = None,
        payment_expiration: float = None,
        payment_method_type: str = None,
        payment_method_type_categories: List[str] = None,
        payment_method_types_exclude: List[str] = None,
        payment_method_types_include: List[str] = None,
        timestamp: float = None,
    ):
        """V1CheckoutBody

        :param account_funding_transaction: Details of an account funding transaction (AFT), which transfers funds from a card to a cardholder's wallet., defaults to None
        :type account_funding_transaction: dict, optional
        :param amount: The amount of the payment, in units of the currency defined in currency. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015. If the amount is a whole number, use an integer and not a decimal.
        :type amount: float
        :param cancel_checkout_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type cancel_checkout_url: str, optional
        :param cart_items: Describes the cart items that the customer is purchasing. These items are displayed at the checkout page., defaults to None
        :type cart_items: HostedPageAdditionalResponseCartItems, optional
        :param complete_checkout_url: URL where the customer is redirected after pressing Finish to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_checkout_url: str, optional
        :param country: country
        :type country: str
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer.
        :type currency: str
        :param customer: Description of the payment transaction. To display the description, set display_description to true in custom_elements., defaults to None
        :type customer: str, optional
        :param escrow: Determines whether the payment is held in escrow for later release., defaults to None
        :type escrow: bool, optional
        :param escrow_release_days: Determines the number of days after creation of the payment that funds are released from escrow. Funds are released at 5:00 pm GMT on the day indicated. Integer, range: 1-90., defaults to None
        :type escrow_release_days: float, optional
        :param id_: ID of the Rapyd checkout page. String starting with **checkout_**., defaults to None
        :type id_: str, optional
        :param merchant_main_button: A string that represents the text on the main Call to Action (CTA) button. One of the following:\<BR\>* place_your_order - Place Your Order.\<BR\>* pay - Pay.\<BR\>* pay_now - Pay Now.\<BR\>* make_payment - Make Payment.\<BR\>* purchase - Purchase.\<BR\>* buy - Buy.\<BR\>* donate - Donate.\<BR\>* To configure this button, use the Client Portal., defaults to None
        :type merchant_main_button: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
        :type merchant_terms: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when one or both of the following fields is unset: \<BR\>* `cancel_url` \<BR\>* `complete_url`.\<BR\> To configure this field, use the Client Portal, defaults to None
        :type merchant_website: str, optional
        :param custom_elements: Description of the payment transaction., defaults to None
        :type custom_elements: HostedPageAdditionalResponseCustomElements, optional
        :param page_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
        :type page_expiration: float, optional
        :param payment: Collects money from a payment method and deposits it into one or more Rapyd Wallets, defaults to None
        :type payment: Payment, optional
        :param payment_expiration: Length of time for the payment to be completed after it is created, measured in seconds. When both expiration and payment_expiration are set, the payment expires at the earlier time., defaults to None
        :type payment_expiration: float, optional
        :param payment_method_type: The type of the payment method. For example, **it_visa_card**, defaults to None
        :type payment_method_type: str, optional
        :param payment_method_type_categories: A list of the categories of payment method that are supported on the checkout page. The categories appear on the page in the order provided, defaults to None
        :type payment_method_type_categories: List[str], optional
        :param payment_method_types_exclude: List of payment methods that are excluded from display on the checkout page., defaults to None
        :type payment_method_types_exclude: List[str], optional
        :param payment_method_types_include: List of payment methods that are displayed on the checkout page. The payment methods appear on the page in the order provided., defaults to None
        :type payment_method_types_include: List[str], optional
        :param timestamp: Time of creation of the checkout page, in Unix time., defaults to None
        :type timestamp: float, optional
        """
        self.account_funding_transaction = account_funding_transaction
        self.amount = amount
        self.cancel_checkout_url = self._define_str(
            "cancel_checkout_url", cancel_checkout_url, nullable=True
        )
        self.cart_items = self._define_object(
            cart_items, HostedPageAdditionalResponseCartItems
        )
        self.complete_checkout_url = self._define_str(
            "complete_checkout_url", complete_checkout_url, nullable=True
        )
        self.country = self._define_str(
            "country",
            country,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.currency = self._define_str(
            "currency",
            currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.escrow = escrow
        self.escrow_release_days = self._define_number(
            "escrow_release_days", escrow_release_days, nullable=True
        )
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_main_button = self._define_str(
            "merchant_main_button", merchant_main_button, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.custom_elements = self._define_object(
            custom_elements, HostedPageAdditionalResponseCustomElements
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.payment = self._define_object(payment, Payment)
        self.payment_expiration = self._define_number(
            "payment_expiration", payment_expiration, nullable=True
        )
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.payment_method_type_categories = payment_method_type_categories
        self.payment_method_types_exclude = payment_method_types_exclude
        self.payment_method_types_include = payment_method_types_include
        self.timestamp = self._define_number("timestamp", timestamp, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .card_transaction import CardTransaction


@JsonMap({"id_": "id"})
class InlineResponse200_29Data(BaseModel):
    """InlineResponse200_29Data

    :param bank_account: Issued virtual account., defaults to None
    :type bank_account: dict, optional
    :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
    :type country: str, optional
    :param currency: Currency of the account. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param description: Description of the account., defaults to None
    :type description: str, optional
    :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**., defaults to None
    :type ewallet: str, optional
    :param funding_instructions: Issued virtual account funding instructions., defaults to None
    :type funding_instructions: dict, optional
    :param id_: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
    :type id_: str, optional
    :param merchant_reference_id: ID of this account, as provided by the merchant. Limited to 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param status: Issued virtual account statuts., defaults to None
    :type status: str, optional
    :param transactions: Issued virtual account transactions., defaults to None
    :type transactions: List[CardTransaction], optional
    :param requested_currency: *Currency received by the virtual account after conversion. Uppercase. One of the following: AUD - Australian Dollar EUR - Euro GBP - Pound Sterling HKD - Hong Kong Dollar SGD - Singapore Dollar USD - US Dollar When not specified, the funds appear in the wallet’s currency account for the currency of the transaction.*, defaults to None
    :type requested_currency: str, optional
    """

    def __init__(
        self,
        bank_account: dict = None,
        country: str = None,
        currency: str = None,
        description: str = None,
        ewallet: str = None,
        funding_instructions: dict = None,
        id_: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
        status: str = None,
        transactions: List[CardTransaction] = None,
        requested_currency: str = None,
    ):
        """InlineResponse200_29Data

        :param bank_account: Issued virtual account., defaults to None
        :type bank_account: dict, optional
        :param country: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
        :type country: str, optional
        :param currency: Currency of the account. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param description: Description of the account., defaults to None
        :type description: str, optional
        :param ewallet: ID of the wallet the bank account number is issued to. String starting with **ewallet_**., defaults to None
        :type ewallet: str, optional
        :param funding_instructions: Issued virtual account funding instructions., defaults to None
        :type funding_instructions: dict, optional
        :param id_: The country that the account number will be associated with. Two-letter ISO 3166-1 ALPHA-2 code. Uppercase., defaults to None
        :type id_: str, optional
        :param merchant_reference_id: ID of this account, as provided by the merchant. Limited to 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param status: Issued virtual account statuts., defaults to None
        :type status: str, optional
        :param transactions: Issued virtual account transactions., defaults to None
        :type transactions: List[CardTransaction], optional
        :param requested_currency: *Currency received by the virtual account after conversion. Uppercase. One of the following: AUD - Australian Dollar EUR - Euro GBP - Pound Sterling HKD - Hong Kong Dollar SGD - Singapore Dollar USD - US Dollar When not specified, the funds appear in the wallet’s currency account for the currency of the transaction.*, defaults to None
        :type requested_currency: str, optional
        """
        self.bank_account = bank_account
        self.country = self._define_str("country", country, nullable=True)
        self.currency = self._define_str("currency", currency, nullable=True)
        self.description = self._define_str("description", description, nullable=True)
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.funding_instructions = funding_instructions
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.status = self._define_str("status", status, nullable=True)
        self.transactions = self._define_list(transactions, CardTransaction)
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class AccountDepositBody(BaseModel):
    """AccountDepositBody

    :param amount: Amount of the transaction. Decimal.
    :type amount: float
    :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field.
    :type currency: str
    :param ewallet: ID of the Rapyd Wallet. String starting with **ewallet_**.
    :type ewallet: dict
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    """

    def __init__(
        self, amount: float, currency: str, ewallet: dict, metadata: dict = None
    ):
        """AccountDepositBody

        :param amount: Amount of the transaction. Decimal.
        :type amount: float
        :param currency: Three-letter ISO 4217 code for the currency used in the `amount` field.
        :type currency: str
        :param ewallet: ID of the Rapyd Wallet. String starting with **ewallet_**.
        :type ewallet: dict
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        """
        self.amount = amount
        self.currency = currency
        self.ewallet = ewallet
        self.metadata = metadata



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .entity_type_verify import EntityTypeVerify


@JsonMap({})
class InlineResponse200_87Data(BaseModel):
    """InlineResponse200_87Data

    :param application_type: Code for the type of application. String starting with typ_., defaults to None
    :type application_type: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param entity_type: entity_type, defaults to None
    :type entity_type: EntityTypeVerify, optional
    """

    def __init__(
        self,
        application_type: str = None,
        country: str = None,
        entity_type: EntityTypeVerify = None,
    ):
        """InlineResponse200_87Data

        :param application_type: Code for the type of application. String starting with typ_., defaults to None
        :type application_type: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param entity_type: entity_type, defaults to None
        :type entity_type: EntityTypeVerify, optional
        """
        self.application_type = self._define_str(
            "application_type", application_type, nullable=True
        )
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="^(A(D|E|F|G|I|L|M|N|O|R|S|T|Q|U|W|X|Z)|B(A|B|D|E|F|G|H|I|J|L|M|N|O|R|S|T|V|W|Y|Z)|C(A|C|D|F|G|H|I|K|L|M|N|O|R|U|V|X|Y|Z)|D(E|J|K|M|O|Z)|E(C|E|G|H|R|S|T)|F(I|J|K|M|O|R)|G(A|B|D|E|F|G|H|I|L|M|N|P|Q|R|S|T|U|W|Y)|H(K|M|N|R|T|U)|I(D|E|Q|L|M|N|O|R|S|T)|J(E|M|O|P)|K(E|G|H|I|M|N|P|R|W|Y|Z)|L(A|B|C|I|K|R|S|T|U|V|Y)|M(A|C|D|E|F|G|H|K|L|M|N|O|Q|P|R|S|T|U|V|W|X|Y|Z)|N(A|C|E|F|G|I|L|O|P|R|U|Z)|OM|P(A|E|F|G|H|K|L|M|N|R|S|T|W|Y)|QA|R(E|O|S|U|W)|S(A|B|C|D|E|G|H|I|J|K|L|M|N|O|R|T|V|Y|Z)|T(C|D|F|G|H|J|K|L|M|N|O|R|T|V|W|Z)|U(A|G|M|S|Y|Z)|V(A|C|E|G|I|N|U)|W(F|S)|Y(E|T)|Z(A|M|W))$",
        )
        self.entity_type = (
            self._enum_matching(entity_type, EntityTypeVerify.list(), "entity_type")
            if entity_type
            else None
        )
